/*-
 * Copyright (c) 2003, Patrick MARIE <mycroft@virgaria.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>

#include <net/if.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip_fw.h>

#include <syslog.h>

extern int      bg;

int
log_info(char *fmt,...)
{
	va_list         ap;

	va_start(ap, fmt);

	if (bg)
		vsyslog(LOG_INFO, fmt, ap);
	else
		vprintf(fmt, ap);
	va_end(ap);

	return 0;
}

int
log_warn(char *fmt,...)
{
	va_list         ap;

	va_start(ap, fmt);

	if (bg)
		vsyslog(LOG_ALERT, fmt, ap);
	else
		vprintf(fmt, ap);
	va_end(ap);

	return (0);
}

int
log_debug(char *fmt,...)
{
	va_list         ap;

	if (bg)
		return -1;

	va_start(ap, fmt);
	vprintf(fmt, ap);
	va_end(ap);

	return (0);
}

int
checkunvalidity(const char *str)
{
	int             i;

	for (i = 0; *(str + i); i++) {
		if (*(str + i) == '#')
			return 1;
		if (*(str + i) != ' ')
			break;
	}

	return 0;
}

char           *
clean_string(const char *str)
{
	char           *tmp, *savetmp, *final;
	int             i, len;

	tmp = strdup(str);
	savetmp = tmp;

	while (*tmp == ' ')
		tmp++;

	len = strlen(tmp);

	for (i = 0; i < len; i++)
		if (*(tmp + i) == '\n' || *(tmp + i) == '\r')
			*(tmp + i) = ' ';

	for (i = strlen(tmp); *(tmp + i - 1) == ' ' && i > 0; i--)
		*(tmp + i - 1) = 0;

	if (strlen(tmp) == 0)
		final = NULL;
	else
		final = strdup(tmp);

	free(savetmp);

	return (final);
}

/* Code from libalias */
ipfw_insn      *
next_cmd(ipfw_insn * cmd)
{
	cmd += F_LEN(cmd);
	bzero(cmd, sizeof(*cmd));
	return cmd;
}

ipfw_insn      *
fill_cmd(ipfw_insn * cmd, enum ipfw_opcodes opcode, int size,
	 int flags, u_int16_t arg)
{
	cmd->opcode = opcode;
	cmd->len = ((cmd->len | flags) & (F_NOT | F_OR)) | (size & F_LEN_MASK);
	cmd->arg1 = arg;
	return next_cmd(cmd);
}

ipfw_insn      *
fill_ip(ipfw_insn * cmd1, enum ipfw_opcodes opcode, u_int32_t addr)
{
	ipfw_insn_ip   *cmd = (ipfw_insn_ip *) cmd1;

	cmd->addr.s_addr = addr;
	return fill_cmd(cmd1, opcode, F_INSN_SIZE(ipfw_insn_u32), 0, 0);
}

ipfw_insn      *
fill_one_port(ipfw_insn * cmd1, enum ipfw_opcodes opcode, u_int16_t port)
{
	ipfw_insn_u16  *cmd = (ipfw_insn_u16 *) cmd1;

	cmd->ports[0] = cmd->ports[1] = port;
	return fill_cmd(cmd1, opcode, F_INSN_SIZE(ipfw_insn_u16), 0, 0);
}
